==> trircd-release-5.1.7-r.tar.gz: trircd-release-5.1.7-r/chanmodes/cm_flood.c <==
/************************************************************************                  
 *   IRC - Internet Relay Chat, chanmodes/cm_flood.c
 *
 *   Copyright (C) 2002 TR-IRCD Development
 *   Copyright (C) 1990 Jarkko Oikarinen and
 *                      University of Oulu, Co Center
 *                      
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2, or (at your option) 
 *   any later version.
 *   
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of 
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 *   GNU General Public License for more details.
 *   
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  
 */

#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "channel.h"
#include "h.h"
#include "msg.h"
#include "hook.h"
#include "s_conf.h"
#include "chanmode.h"

static int set_flood(int adl, aChannel *chptr, int nmodes,
		     int *argnum, int *pidx, int *mbix, char *mbuf, char *pbuf,
		     aClient *cptr, aClient *sptr, int parc, char **parv)
{

    char *pptr;			/* temporary paramater pointer */
    char *morig = mbuf;		/* beginning of mbuf */
    char *p = NULL;
    char *c = NULL;

    int prelen = strlen(cptr->name) + strlen(chptr->chname) + 16;

    int fargnum = *argnum;
    int fmbix = *mbix;
    int fpidx = *pidx;

    int i, j;
    char tmp[16];		/* temporary buffer */

    if (adl == CMODE_DEL) {
	if ((prelen + (mbuf - morig) + fpidx + 1) > REALMODEBUFLEN)
	    return nmodes;
	mbuf[fmbix++] = 'f';
	chptr->mode.mode &= ~MODE_FLOOD;
	chptr->mode.lines = 0;
	chptr->mode.intime = 0;
	nmodes++;

	*mbix = fmbix;

    }

    if (adl == CMODE_ADD) {
	if (parv[fargnum] == NULL) {
	    send_me_numeric(sptr, ERR_NEEDMOREPARAMS, MSG_MODE);
	    return nmodes;
	}

	if ((c = strchr(parv[fargnum], ':')) == NULL) {
	    send_me_numeric(sptr, ERR_NEEDMOREPARAMS, MSG_MODE);
	    return nmodes;
	}

	/*
	 * if we're going to overflow our mode buffer,
	 * * drop the change instead
	 */

	if ((prelen + (mbuf - morig) + fpidx + 16) > REALMODEBUFLEN) {
	    fargnum++;
	    return nmodes;
	}

	i = atoi(strtoken(&p, parv[fargnum], ":"));
	j = atoi(++c);

	if ((i < 1) || (j < 1)) {
	    fargnum++;
	    *argnum = fargnum;
	    return nmodes;
	}

	ircsprintf(tmp, "%d:%d", i, j);
	chptr->mode.mode |= MODE_FLOOD;
	chptr->mode.lines = i;
	chptr->mode.intime = j;
	mbuf[fmbix++] = 'f';

	pptr = tmp;
	if (fpidx)
	    pbuf[fpidx++] = ' ';
	while (*pptr)
	    pbuf[fpidx++] = *pptr++;
	fargnum++;

	*mbix = fmbix;
	*pidx = fpidx;
	*argnum = fargnum;

	nmodes++;
    }
    return nmodes;
}

static int sf_service(int adl, aChannel *chptr, int nmodes,
		      int *argnum, int *pidx, int *mbix, char *mbuf, char *pbuf,
		      aClient *cptr, aClient *sptr, int parc, char **parv)
{
    if (MyClient(sptr))
	send_me_numeric(sptr, ERR_CHANOPRIVSNEEDED, chptr);
    else
	ircstp->is_fake++;
    return nmodes;
}

static int sf_server(int adl, aChannel *chptr, int nmodes,
		     int *argnum, int *pidx, int *mbix, char *mbuf, char *pbuf,
		     aClient *cptr, aClient *sptr, int parc, char **parv)
{
    return set_flood(adl, chptr, nmodes, argnum, pidx, mbix, mbuf, pbuf, cptr, sptr, parc, parv);
}

static int sf_uline(int adl, aChannel *chptr, int nmodes,
		    int *argnum, int *pidx, int *mbix, char *mbuf, char *pbuf,
		    aClient *cptr, aClient *sptr, int parc, char **parv)
{
    return set_flood(adl, chptr, nmodes, argnum, pidx, mbix, mbuf, pbuf, cptr, sptr, parc, parv);
}

static int sf_oper(int adl, aChannel *chptr, int nmodes,
		   int *argnum, int *pidx, int *mbix, char *mbuf, char *pbuf,
		   aClient *cptr, aClient *sptr, int parc, char **parv)
{
    if (IsChanUser(sptr, chptr, CHFL_CHANOP) || IsOperMode(sptr) || IsServer(cptr)) {
        
        return set_flood(adl, chptr, nmodes, argnum, pidx, mbix, mbuf, pbuf, cptr, sptr, parc, parv);
    } else
	send_me_numeric(sptr, ERR_CHANOPRIVSNEEDED, chptr);
    return nmodes;
}

static int sf_user(int adl, aChannel *chptr, int nmodes,
		   int *argnum, int *pidx, int *mbix, char *mbuf, char *pbuf,
		   aClient *cptr, aClient *sptr, int parc, char **parv)
{
    if (IsChanUser(sptr, chptr, CHFL_CHANOP))
	return set_flood(adl, chptr, nmodes, argnum, pidx, mbix, mbuf, pbuf,
			 cptr, sptr, parc, parv);
    else
	send_me_numeric(sptr, ERR_CHANOPRIVSNEEDED, chptr);
    return nmodes;
}

static struct ChanMode mode_flood[] = {
    {MODE_FLOOD, 1, 0, sf_user, sf_oper, sf_uline, sf_server, sf_service}
};

#ifndef STATIC_MODULES
int _persistent = 1;
void _modinit(void)
#else
void flood_modinit(void)
#endif
{
    if (ChannelConf.floodmode)
	modetab[(int) 'f'] = mode_flood[0];
}

==> trircd-release-5.7.3-r.tar.gz: trircd-release-5.7.3-r/chanmodes/cm_flood.c <==
/************************************************************************                  
 *   IRC - Internet Relay Chat, chanmodes/cm_flood.c
 *
 *   Copyright (C)2000-2003 TR-IRCD Development
 *   Copyright (C) 1990 Jarkko Oikarinen and
 *                      University of Oulu, Co Center
 *                      
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2, or (at your option) 
 *   any later version.
 *   
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of 
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 *   GNU General Public License for more details.
 *   
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  
 */

#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "channel.h"
#include "h.h"
#include "msg.h"
#include "hook.h"
#include "s_conf.h"
#include "chanmode.h"

static int set_flood(int adl, aChannel *chptr, int nmodes,
		     int *argnum, int *pidx, int *mbix, char *mbuf, char *pbuf,
		     aClient *cptr, aClient *sptr, int parc, char **parv)
{

    char *pptr;			/* temporary paramater pointer */
    char *morig = mbuf;		/* beginning of mbuf */
    char *p = NULL;
    char *c = NULL;

    int prelen = strlen(cptr->name) + strlen(chptr->chname) + 16;

    int fargnum = *argnum;
    int fmbix = *mbix;
    int fpidx = *pidx;

    int i, j;
    char tmp[16];		/* temporary buffer */

    if (adl == CMODE_DEL) {
	if ((prelen + (mbuf - morig) + fpidx + 1) > REALMODEBUFLEN)
	    return nmodes;
	mbuf[fmbix++] = 'f';
	chptr->mode.mode &= ~MODE_FLOOD;
	chptr->mode.lines = 0;
	chptr->mode.intime = 0;
	nmodes++;

	*mbix = fmbix;

    }

    if (adl == CMODE_ADD) {
	if (parv[fargnum] == NULL) {
	    send_me_numeric(sptr, ERR_NEEDMOREPARAMS, MSG_MODE);
	    return nmodes;
	}

	if ((c = strchr(parv[fargnum], ':')) == NULL) {
	    send_me_numeric(sptr, ERR_NEEDMOREPARAMS, MSG_MODE);
	    return nmodes;
	}

	/*
	 * if we're going to overflow our mode buffer,
	 * * drop the change instead
	 */

	if ((prelen + (mbuf - morig) + fpidx + 16) > REALMODEBUFLEN) {
	    fargnum++;
	    return nmodes;
	}

	i = atoi(strtoken(&p, parv[fargnum], ":"));
	j = atoi(++c);

	if ((i < 1) || (j < 1)) {
	    fargnum++;
	    *argnum = fargnum;
	    return nmodes;
	}

	ircsprintf(tmp, "%d:%d", i, j);
	chptr->mode.mode |= MODE_FLOOD;
	chptr->mode.lines = i;
	chptr->mode.intime = j;
	mbuf[fmbix++] = 'f';

	pptr = tmp;
	if (fpidx)
	    pbuf[fpidx++] = ' ';
	while (*pptr)
	    pbuf[fpidx++] = *pptr++;
	fargnum++;

	*mbix = fmbix;
	*pidx = fpidx;
	*argnum = fargnum;

	nmodes++;
    }
    return nmodes;
}

static int sf_service(int adl, aChannel *chptr, int nmodes,
		      int *argnum, int *pidx, int *mbix, char *mbuf, char *pbuf,
		      aClient *cptr, aClient *sptr, int parc, char **parv)
{
    if (MyClient(sptr))
	send_me_numeric(sptr, ERR_CHANOPRIVSNEEDED, chptr);
    else
	ircstp->is_fake++;
    return nmodes;
}

static int sf_server(int adl, aChannel *chptr, int nmodes,
		     int *argnum, int *pidx, int *mbix, char *mbuf, char *pbuf,
		     aClient *cptr, aClient *sptr, int parc, char **parv)
{
    return set_flood(adl, chptr, nmodes, argnum, pidx, mbix, mbuf, pbuf, cptr, sptr, parc, parv);
}

static int sf_uline(int adl, aChannel *chptr, int nmodes,
		    int *argnum, int *pidx, int *mbix, char *mbuf, char *pbuf,
		    aClient *cptr, aClient *sptr, int parc, char **parv)
{
    return set_flood(adl, chptr, nmodes, argnum, pidx, mbix, mbuf, pbuf, cptr, sptr, parc, parv);
}

static int sf_oper(int adl, aChannel *chptr, int nmodes,
		   int *argnum, int *pidx, int *mbix, char *mbuf, char *pbuf,
		   aClient *cptr, aClient *sptr, int parc, char **parv)
{
    if (IsChanUser(sptr, chptr, CHFL_CHANOP) || IsOperMode(sptr) || IsServer(cptr)) {
        
        return set_flood(adl, chptr, nmodes, argnum, pidx, mbix, mbuf, pbuf, cptr, sptr, parc, parv);
    } else
	send_me_numeric(sptr, ERR_CHANOPRIVSNEEDED, chptr);
    return nmodes;
}

static int sf_user(int adl, aChannel *chptr, int nmodes,
		   int *argnum, int *pidx, int *mbix, char *mbuf, char *pbuf,
		   aClient *cptr, aClient *sptr, int parc, char **parv)
{
    if (IsChanUser(sptr, chptr, CHFL_CHANOP))
	return set_flood(adl, chptr, nmodes, argnum, pidx, mbix, mbuf, pbuf,
			 cptr, sptr, parc, parv);
    else
	send_me_numeric(sptr, ERR_CHANOPRIVSNEEDED, chptr);
    return nmodes;
}

static struct ChanMode mode_flood[] = {
    {MODE_FLOOD, 1, MFLG_PARAMDBL, CMTYPE_PARAMETRIC | CMTYPE_REMWOPARA, 
     sf_user, sf_oper, sf_uline, sf_server, sf_service}
};

#ifndef STATIC_MODULES
int _persistent = 1;
void _modinit(void)
#else
void flood_modinit(void)
#endif
{
    modetab[(int) 'f'] = mode_flood[0];
    GeneralOpts.lists_created = 0;
}
